package com.ibm.oti.shared;

/*
 * Licensed Materials - Property of IBM,
 *     Copyright IBM Corp. 2010, 2011  All Rights Reserved
 */

/**
 * SharedClassCacheInfo stores information about a shared class cache and
 * provides methods to retrieve that information.
 * <p>
 */
public class SharedClassCacheInfo {

	/**
	 * Specifies Java 5 cache.
	 */
	static final public int JVMLEVEL_JAVA5 = 1;
	/**
	 * Specifies Java 6 cache.
	 */
	static final public int JVMLEVEL_JAVA6 = 2;
	/**
	 * Specifies Java 7 cache.
	 */
	static final public int JVMLEVEL_JAVA7 = 3;

	/**
	 * Specifies a 32-bit cache.
	 */
	static final public int ADDRESS_MODE_32 = 32;

	/**
	 * Specifies a 64-bit cache.
	 */
	static final public int ADDRESS_MODE_64 = 64;

	private String name;
	private boolean isCompatible;
	private boolean isPersistent;
	private int osShmid;
	private int osSemid;
	private long lastDetach;
	private int modLevel;
	private int addrMode;
	private boolean isCorrupt;
	private long cacheSize;
	private long freeBytes;

	/**
	 * Gets the cache name for the shared class cache.
	 *
	 * @return		name of the shared class cache.
	 */
	public String getCacheName() {
		return name;
	}

	/**
	 * Check the compatibility of the shared class cache with this JVM.
	 *
	 * @return		true if cache is compatible with this JVM, false otherwise.
	 */
	public boolean isCacheCompatible() {
		return isCompatible;
	}

	/**
	 * Check if the shared class cache is persistent.
	 *
	 * @return		true if cache is persistent, false otherwise.
	 */
	public boolean isCachePersistent() {
		return isPersistent;
	}

	/**
	 * Get the OS shared memory id associated with the shared class cache.
	 *
	 * @return		a valid value if cache is non-persistent and shared memory id is available, else -1.
	 */
	public int getOSshmid() {
		return osShmid;
	}

	/**
	 * Get the OS semaphore id associated with the shared class cache.
	 *
	 * @return		a valid value if cache is non-persistent and semaphore id is available, else -1.
	 */
	public int getOSsemid() {
		return osSemid;
	}

	/**
	 * Get the time when the shared class cache was last detached.
	 *
	 * @return		Date
	 */
	public java.util.Date getLastDetach() {
		return new java.util.Date(lastDetach);
	}

	/**
	 * Get the JVM level for the shared class cache.
	 *
	 * @return		either {@link SharedClassCacheInfo#JVMLEVEL_JAVA5},
	 * 				{@link SharedClassCacheInfo#JVMLEVEL_JAVA6} or
	 * 				{@link SharedClassCacheInfo#JVMLEVEL_JAVA7}
	 */
	public int getCacheJVMLevel() {
		return modLevel;
	}

	/**
	 * Get the address mode for the shared class cache.
	 *
	 * @return		either {@link SharedClassCacheInfo#ADDRESS_MODE_32} or
	 * 				{@link SharedClassCacheInfo#ADDRESS_MODE_64}
	 */
	public int getCacheAddressMode() {
		return addrMode;
	}

	/**
	 * Check if the shared class cache is corrupt.
	 *
	 * @return		true if the cache is corrupt, false otherwise.
	 */
	public boolean isCacheCorrupt() {
		return isCorrupt;
	}

	/**
	 * Get total usable shared class cache size. Returns -1 if cache is incompatible.
	 *
	 * @return		number of usable bytes in cache.
	 */
	public long getCacheSize() {
		return cacheSize;
	}

	/**
	 * Get the amount of free bytes in shared class cache. Returns -1 if cache is incompatible.
	 *
	 * @return		long
	 */
	public long getCacheFreeBytes() {
		return freeBytes;
	}

	/**
	 * Constructor to create an object of SharedClassCacheInfo using the parameter specified.
	 *
	 * @param		name			shared class cache name
	 * @param		isCompatible	compatibility of the cache
	 * @param		isPersistent	cache type
	 * @param		osShmid			OS shared memory id for the cache
	 * @param		osSemid			OS semaphore id for the cache
	 * @param		lastDetach		time of last detach
	 * @param		modLevel		JVM level of the cache
	 * @param		addrMode		address mode of the cache
	 * @param		isCorrupt
	 * @param		cacheSize		size of cache in bytes
	 * @param		freeBytes		free bytes in cache
	 */
	SharedClassCacheInfo(String name,
						boolean isCompatible,
						boolean isPersistent,
						int osShmid,
						int osSemid,
						long lastDetach,
						int modLevel,
						int addrMode,
						boolean isCorrupt,
						long cacheSize,
						long freeBytes)
	{
		this.name = name;
		this.isCompatible = isCompatible;
		this.isPersistent = isPersistent;
		this.osShmid = osShmid;
		this.osSemid = osSemid;
		this.lastDetach = lastDetach;
		this.modLevel = modLevel;
		this.addrMode = addrMode;
		this.isCorrupt = isCorrupt;
		this.cacheSize = cacheSize;
		this.freeBytes = freeBytes;
	}
}
