package com.ibm.oti.shared;

/*
 * Licensed Materials - Property of IBM,
 *     Copyright IBM Corp. 2010  All Rights Reserved
 */

import java.util.*;

/**
 * Shared Class Utilities APIs to get information about all shared class caches in a directory and
 * to destroy a particular shared class cache.
 * <p>
 *
 * @see SharedClassCacheInfo
 *
 */
public class SharedClassUtilities {

	/**
	 * Value to be passed to <code>flags</code> parameter of {@link SharedClassUtilities#getSharedCacheInfo} method.
	 */
	static final public int NO_FLAGS = 0;
	/**
	 * Use the platform dependent default value as the cache type.
	 */
	static final public int PERSISTENCE_DEFAULT = 0;
	/**
	 * Specifies a persistent cache.
	 */
	static final public int PERSISTENT = 1;
	/**
	 * Specifies a non-persistent cache.
	 */
	static final public int NONPERSISTENT = 2;
	/**
	 * Returned by {@link SharedClassUtilities#destroySharedCache} to indicate either no cache exists
	 * or the method has successfully destroyed caches of all generations.
	 */
	static final public int DESTROYED_ALL_CACHE = 0;
	/**
	 * Returned by {@link SharedClassUtilities#destroySharedCache} to indicate the method failed to destroy any cache.
	 */
	static final public int DESTROYED_NONE = -1;
	/**
	 * Returned by {@link SharedClassUtilities#destroySharedCache} to indicate the method has failed to destroy the
	 * current generation cache.
	 */
	static final public int DESTROY_FAILED_CURRENT_GEN_CACHE = -2;
	/**
	 * Returned by {@link SharedClassUtilities#destroySharedCache} to indicate the method has failed to destroy one
	 * or more older generation cache, and either a current generation cache does not exist or is successfully destroyed.
	 */
	static final public int DESTROY_FAILED_OLDER_GEN_CACHE = -3;
	/**
	 * Returned by native methods to indicate shared class utilities support is disabled.
	 */
	static final private int SHARED_CLASSES_UTILITIES_DISABLED = -255;

	private static native void init();
	private static native int getSharedCacheInfoImpl(String cacheDir, int flags, boolean useCommandLineValues, List<SharedClassCacheInfo> arrayList);
	private static native int destroySharedCacheImpl(String cacheDir, int persistence, String cacheName, boolean useCommandLineValues);

	static {
		init();
	}

	/**
	 * Iterate through all shared class caches present in the given directory and return their information in
	 * a {@link List}.
	 * <p>
	 * If <code>useCommandLineValues</code> is true then use the command line value as the directory to search in.
	 * If the command line value is not available, use the platform dependent default value.
	 * If <code>useCommandLineValues</code> is false, then use <code>cacheDir</code> as the directory to search in.
	 * <code>cacheDir</code> can be <code>null</code>. In such case, use the platform dependent default value.
	 *
	 * @param		cacheDir Absolute path of the directory to look for the shared class caches
	 * @param		flags Reserved for future use. Always pass {@link SharedClassUtilities#NO_FLAGS}
	 * @param		useCommandLineValues Use command line values instead of using parameter values
	 *
	 * @return		List of {@link SharedClassCacheInfo} corresponding to shared class caches which are present
	 * in the specified directory, <code>null</code> on failure.
	 *
	 * @throws		IllegalStateException
	 * 					if shared classes is disabled for this JVM (i.e. -Xshareclasses:none is present).
	 * 				IllegalArgumentException
	 * 					if <code>flags<code> is not a valid value.
	 */
	static public List<SharedClassCacheInfo> getSharedCacheInfo(String cacheDir, int flags, boolean useCommandLineValues) {
		int retVal;
		// K0553 = parameter {0} has invalid value
		if (flags != NO_FLAGS) {
			throw new IllegalArgumentException(com.ibm.oti.util.Msg.getString("K0553", "\"flags\""));	//$NON-NLS-1$
		}
		ArrayList<SharedClassCacheInfo> arrayList = new ArrayList<SharedClassCacheInfo>();
		retVal = getSharedCacheInfoImpl(cacheDir, flags, useCommandLineValues, arrayList);
		if (-1 == retVal) {
			return null;
		} else if(SHARED_CLASSES_UTILITIES_DISABLED == retVal) {
			// K0557 = Shared Classes support is disabled with -Xshareclasses:none
			throw new IllegalStateException(com.ibm.oti.util.Msg.getString("K0557"));
		}
		return arrayList;
	}

	/**
	 * Destroy a named shared class cache of a given type in a given directory.
	 * <p>
	 * If <code>useCommandLineValues</code> is true then use the command line value to get the shared class cache name,
	 * its type and its directory. If any of these is not available, then use the default value.
	 * If <code>useCommandLineValues</code> is false, then use <code>cacheDir</code>, <code>persistence</code> and
	 * <code>cacheName</code> to identify the cache to be destroyed. To accept the default value for <code>cacheDir</code>
	 * or <code>cacheName</code>, specify the parameter with a <code>null</code> value.
	 * <p>
	 * Return value of this method depends on the status of existing current and older generation caches.
	 * <ul>
	 * <li>If it fails to destroy any existing cache with the given name, it returns
	 * {@link SharedClassUtilities#DESTROYED_NONE}.
	 * <li>If no cache exists or it is able to destroy all existing caches of all generations, it returns
	 * {@link SharedClassUtilities#DESTROYED_ALL_CACHE}.<br>
	 * <li>If it fails to destroy an existing current generation cache, irrespective of the state of older generation
	 * cache(s), it returns {@link SharedClassUtilities#DESTROY_FAILED_CURRENT_GEN_CACHE}.<br>
	 * <li>If it fails to destroy one or more older generation cache and either a current generation cache does not
	 * exist or is successfully destroyed, it returns {@link SharedClassUtilities#DESTROY_FAILED_OLDER_GEN_CACHE}.<br>
	 * 	</ul>
	 *
	 * @param		cacheDir Absolute path of the directory where the shared class cache is present
	 * @param		persistence Type of the cache. One of:
	 * <ul>
	 * <li>{@link SharedClassUtilities#PERSISTENCE_DEFAULT}
	 * <li>{@link SharedClassUtilities#PERSISTENT}
	 * <li>{@link SharedClassUtilities#NONPERSISTENT}
	 * </ul>
	 * @param		cacheName Name of the cache to be deleted
	 * @param		useCommandLineValues Use command line values instead of using parameter values
	 *
	 * @return		Returns one of the following values:
	 * 				<ul>
	 * 				<li>{@link SharedClassUtilities#DESTROYED_ALL_CACHE}
	 * 				<li>{@link SharedClassUtilities#DESTROYED_NONE}
	 * 				<li>{@link SharedClassUtilities#DESTROY_FAILED_CURRENT_GEN_CACHE}
	 * 				<li>{@link SharedClassUtilities#DESTROY_FAILED_OLDER_GEN_CACHE}
	 * 				</ul>
	 *
	 * @throws		IllegalStateException
	 * 					if shared classes is disabled for this JVM (i.e. -Xshareclasses:none is present).
	 * 				IllegalArgumentException
	 * 					if <code>persistence<code> is not a valid value.
	 */
	static public int destroySharedCache(String cacheDir, int persistence, String cacheName, boolean useCommandLineValues) {
		int retVal = -1;

		/*
		 * check if persistence has a valid value only if we are not using command line values.
		 */
		// K0553 = parameter {0} has invalid value
		if ((useCommandLineValues == false) && (persistence != PERSISTENCE_DEFAULT) && (persistence != PERSISTENT) && (persistence != NONPERSISTENT)) {
			throw new IllegalArgumentException(com.ibm.oti.util.Msg.getString("K0553", "\"persistence\""));	//$NON-NLS-1$
		}

		retVal = destroySharedCacheImpl(cacheDir, persistence, cacheName, useCommandLineValues);
		if (SHARED_CLASSES_UTILITIES_DISABLED == retVal) {
			// K0557 = Shared Classes support is disabled with -Xshareclasses:none
			throw new IllegalStateException(com.ibm.oti.util.Msg.getString("K0557"));
		}

		return retVal;
	}
}

